@@ -13,7 +13,7 @@ from datetime import datetime
 import statistics
 from botocore.exceptions import ClientError
 import numbers
-from matplotlib.colors import LogNorm, Normalize
+from matplotlib.colors import LogNorm, Normalize, SymLogNorm
 
 
 def days_diff(d1, d2):
@@ -38,6 +38,7 @@ def accuracyMetric(target, reported):
 
 def accuracy_metrics(vals, truth_val, expected_dt):
     """
+    normalized accuracy = average( 1 - acc_i / truth_i) where x_i is not null
 
     :param vals:
     :param expected_dt:
@@ -129,9 +130,18 @@ def time_till_metrics(vals, truth_val, expected_dt):
 
 
 def validity_nondecreasing_step(series, truth_df, expected_df, *args, **kwargs):
+    """
+    - 1 + (2/ (1 + e^(- 5 * num increasing updates / possible updates)
+    :param series:
+    :param truth_df:
+    :param expected_df:
+    :param args:
+    :param kwargs:
+    :return:
+    """
     truth_max = truth_df[expected_df.index < series.name].iloc[:, 0].max()
     num_expected_updates = len(expected_df[expected_df.index < series.name].iloc[:, 0].unique())
-    denom = (min(num_expected_updates, truth_max) / 5)
+    denom = (min(num_expected_updates, truth_max) / 5)  # min represents the possible updates
     if denom == 0:
         return None
     coef = -1 / denom
@@ -176,7 +186,11 @@ def validity_nondecreasing_smoothness(series, truth_df, expected_df, *args, **kw
 
 
 def validity_nondecreasing(series, *args, **kwargs):
-    """Percent of sequential reported dates that decrease"""
+    """
+    Percent of changes in sequential observations that increase (within a release)
+    1 - (num of changes that decrease) / (number of changes)
+    where a change is a reported value which is not equal to the previous observation
+    """
     num_decreasing_updates = len([each for each in series[series.notna()].diff() if each < 0])
     num_updates = len([each for each in series[series.notna()].diff() if each != 0])
     if num_updates > 2:
@@ -185,7 +199,16 @@ def validity_nondecreasing(series, *args, **kwargs):
         return 1
 
 
-def consistency_nondecreasing_percent_restatement(series, truth_df, expected_df, *args, **kwargs):
+def consistency_percent_of_release_changes_increasing(series, truth_df, expected_df, *args, **kwargs):
+    """
+    Returns the percent of total abs(changes) in a release that are positive
+    :param series: 
+    :param truth_df: 
+    :param expected_df: 
+    :param args: 
+    :param kwargs: 
+    :return: 
+    """
     # |sum of increases| / |sum of absolute changes|
     increases = sum([diff for diff in series.diff().dropna() if diff > 0])
     changes = sum([abs(diff) for diff in series.diff().dropna()])
@@ -195,20 +218,29 @@ def consistency_nondecreasing_percent_restatement(series, truth_df, expected_df,
         return None
 
 
-def consistency_between_updates(data_df, truth_df, expected_df):
-    updates = data_df.columns
+def consistency_between_releases(data_df, truth_df, expected_df):
+    """
+    Returns a pandas dataframe of percent of release restated from prior release 
+    :param data_df: 
+    :param truth_df: 
+    :param expected_df: 
+    :return: 
+    """
+    releases = data_df.columns
     dct = {}
     last_non_null = None
 
-    for prior, post in zip(updates[:-1], updates[1:]):
+    for prior, post in zip(releases[:-1], releases[1:]):
 
         if len(data_df[prior].notna()) > 0:
             last_non_null = data_df[prior]
+        
         post_nonnulls = data_df[post].notna()
+        
         diff = data_df[post][post_nonnulls] - last_non_null[post_nonnulls]
 
         if len(diff) > 0:
-            dct[post] = {"ConsistencyPercentUpdated": len(diff[diff > 0]) / len(diff)}
+            dct[post] = {"ConsistencyPercentUpdated": len(diff[diff != 0]) / len(diff)}
         else:
             dct[post] = None
 
@@ -221,6 +253,7 @@ def consistency_between_updates(data_df, truth_df, expected_df):
 
 def apply_to_observations(func):
     """
+    Execute a function over the values reported for an observation across releases
 
     :param func:
     :return: func
@@ -246,14 +279,15 @@ def apply_to_observations(func):
     return over_observations
 
 
-def apply_to_updates(func):
+def apply_to_releases(func):
     """
+    Execute a function over the values sequentially across observations in a given release
 
     :param func:
     :return: func
     """
 
-    def over_updates(data_df, truth_df, expected_df, *args, **kwargs):
+    def over_releases(data_df, truth_df, expected_df, *args, **kwargs):
         dct = {}
 
         for update, series in data_df.iteritems():
@@ -267,7 +301,7 @@ def apply_to_updates(func):
 
         return pd.DataFrame.from_dict(dct, orient='columns')
 
-    return over_updates
+    return over_releases
 
 
 def timeliness(data_df, truth_df, expected_df):
@@ -307,7 +341,7 @@ def validity(data_df, truth_df, expected_df):
                 validity_nondecreasing_step(series, truth_df, expected_df),
         }
 
-    func = apply_to_updates(validity_metrics)
+    func = apply_to_releases(validity_metrics)
     return func(data_df, truth_df, expected_df)
 
 
@@ -315,21 +349,21 @@ def consistency(data_df, truth_df, expected_df):
     def consistency_metrics(series, truth_df, expected_df):
         return {
             "ConsistencyNondecreasingPercentRestatement":
-                consistency_nondecreasing_percent_restatement(series, truth_df, expected_df)
+                consistency_percent_of_release_changes_increasing(series, truth_df, expected_df)
         }
 
-    func = apply_to_updates(consistency_metrics)
+    func = apply_to_releases(consistency_metrics)
     df1 = func(data_df, truth_df, expected_df)
     print(type(df1))
     print(df1.shape)
-    df2 = consistency_between_updates(data_df, truth_df, expected_df)
+    df2 = consistency_between_releases(data_df, truth_df, expected_df)
     print(type(df2))
     print(df2.shape)
     return pd.concat([df1, df2])
 
 
 def completeness(data_df, truth_df, expected_df):
-    func = apply_to_updates(completeness_metric)
+    func = apply_to_releases(completeness_metric)
     return func(data_df, truth_df, expected_df)
 
 
@@ -670,8 +704,9 @@ def plot_node(
     truth_df = df_from_s3_csv(prepared_bucket, make_key(truth_file, path))
     image_name = 'truth_plot.png'
     fancy_path = path.replace("/", ": ").replace("_", " ")
-    truth_df.iloc[:, -1].plot(kind="line")
-    plt.title(fancy_path, fontsize=15)
+    truth_df.iloc[:, -1].plot(kind="line", figsize=(14, 14))
+    plt.title(fancy_path, fontsize=20)
+    plt.xticks(rotation=45, ha='right')
     save_plt_to_s3(plt, to_bucket, make_key(image_name, path), format='png')
 
     print('Clearing Image')
@@ -680,9 +715,9 @@ def plot_node(
 
     # plot heatmap
     print("Creating Heatmap of Data")
-    image_name = 'data_heatmap.png'
+    image_name = 'data_heatmap_lognorm.png'
     data_df = df_from_s3_csv(prepared_bucket, make_key(data_file, path))
-    plt.figure(figsize=(12, 12))
+    plt.figure(figsize=(15, 15))
     plt.pcolormesh(data_df, norm=LogNorm(vmin=1))
     len_ind = len(data_df.index)
     len_cols = len(data_df.columns)
@@ -698,14 +733,47 @@ def plot_node(
     plt.ylabel('Reporting Date (Subject)', fontsize=15)
     save_plt_to_s3(plt, to_bucket, make_key(image_name, path), format='png')
 
-    print('Clearing Heatmap')
+    print('Creating non-norm heatmap Heatmap')
+    image_name = 'data_heatmap.png'
+    plt.pcolormesh(data_df)
+    plt.colorbar()
+    save_plt_to_s3(plt, to_bucket, make_key(image_name, path), format='png')
+
+    print('Creating non-norm heatmap Heatmap normalized')
+    image_name = 'data_heatmap_normalized.png'
+    plt.pcolormesh(data_df, norm=Normalize())
+    plt.colorbar()
+    save_plt_to_s3(plt, to_bucket, make_key(image_name, path), format='png')
+    print('Clearing Heatmaps')
     plt.close()
     print('Cleared Heatmap')
 
     print("Creating Heatmap of Differences of Releases")
     image_name = 'data_diff_heatmap.png'
-    plt.figure(figsize=(12, 12))
-    plt.pcolormesh(data_df.diff(axis=1), norm=Normalize())
+    plt.figure(figsize=(15, 15))
+    plt.pcolormesh(data_df.diff(axis=1), norm=SymLogNorm(linthresh=0.5))
+    len_ind = len(data_df.index)
+    len_cols = len(data_df.columns)
+    ynum = max(len_ind // 10, 1)
+    xnum = max(len_cols // 5, 1)
+    yticks = [each for each in range(len_ind) if each % ynum == 0]
+    xticks = [each for each in range(len_cols) if each % xnum == 0]
+    plt.yticks(ticks=[y + .5 for y in yticks], labels=data_df.index[yticks])
+    plt.xticks(ticks=[x + .5 for x in xticks], labels=data_df.columns[xticks], rotation=45, ha='right')
+    plt.colorbar()
+    plt.title(f'{fancy_path} Release Differences', fontsize=15)
+    plt.xlabel('Update', fontsize=15)
+    plt.ylabel('Reporting Date (Subject)', fontsize=15)
+    save_plt_to_s3(plt, to_bucket, make_key(image_name, path), format='png')
+
+    print('Clearing Heatmap of Differences ')
+    plt.close()
+    print('Cleared Heatmap of Differences ')
+
+    print("Creating Heatmap of Differences within Releases")
+    image_name = 'data_diff_heatmap_win_release.png'
+    plt.figure(figsize=(15, 15))
+    plt.pcolormesh(data_df.diff(axis=0), norm=SymLogNorm(linthresh=0.5))
     len_ind = len(data_df.index)
     len_cols = len(data_df.columns)
     ynum = max(len_ind // 10, 1)
@@ -743,7 +811,7 @@ def plot_node(
         cell_text.append([f'{shorten(x)}' for x in row])
 
     plt.figure(linewidth=2,
-               figsize=(12, 7)
+               figsize=(15, 7)
                )
     plt.tight_layout()
     # Add a table at the bottom of the axes
